# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 Max Stewart
# All rights reserved.
#
# This file is part of the TicketValidator plugin for Trac
#
# TicketValidator is free software: you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation, either version 3 of 
# the License, or (at your option) any later version.
#
# TicketValidator is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TicketValidator.  If not, see 
# <http://www.gnu.org/licenses/>.

from trac.core import *
from trac.ticket import ITicketManipulator
from trac.ticket import TicketSystem

class RequiredFieldValidator(Component):
    """Basic ticket validator for required fields"""
    
    implements(ITicketManipulator)
    
    # No @staticmethods! Used as callables below!
    def _is_empty(value):
        """Check if 'value' is empty.
        
        :param value: the value to check
        :type value: object"""
        
        if value is None:
            return False        # field does not exist
        
        if len(value) == 0:
            return True
        
        return False

    def _is_integer(value):
        if value is None:
            return False        # field does not exist

        try:
            int(value)
        except ValueError:
            return True

        return False

    def _is_float(value):
        if value is None:
            return False        # field does not exist

        try:
            float(value)
        except ValueError:
            return True

        return False

    # this list can be extended for more checks
    # format: keyword, check function for field, message tail

    _actionlst = (
            ('required', _is_empty, 'is required'),
            ('integer', _is_integer, 'must be an integer'),
            ('float', _is_float, 'must be a float')
            )

    def _getlabel(self, fieldname):
        """return the field label for given field name if exists"""

        for sect in self.config.sections():
            for (key, val) in self.config.options(sect):
                if fieldname == key:
                    try:
                        return self.config.get(sect, fieldname + '.label')
                    except ConfigParser.NoOptionError:
                        pass

        return fieldname

    def prepare_ticket(self, req, ticket, fields, actions):
        """Not currently called, but should be provided for future
        compatibility."""

    def validate_ticket(self, req, ticket):
        """Make sure required fields for the next state have been 
        filled correctly before the ticket will be entered."""
    
        # get ticket parameters

        parms = (self._get_state(req, ticket), ticket['type'])

        # create list of matches

        errors = []

        for key, value in self.config.options('ticketvalidator'):
            required_fields = [x.strip() for x in value.split(',')]
            patts = key.split('.')

            action = [(fct, msg) for (name, fct, msg) in self._actionlst \
                    if name == patts[-1]]

            if len(action) != 1:
                raise TracError(('illegal attribute "%s" in line\n' \
                        '%s = %s') % (patts[-1], key, value))

            action = action[0]

            # status check, not done if status field is '-'

            for indx in range(len(patts)-1):
                if patts[indx] != '*' and patts[indx] != parms[indx]:
                    break
            else:       # no break, all tests passed
                errors.extend(
                        [(self._getlabel(field_name), action[1])
                            for field_name in required_fields
                            if field_name and \
                                    action[0](ticket[field_name])]
                        )

        return errors
    
    def _get_state(self, req, ticket):
        """Get the state this ticket is going to be in."""
        
        if 'action' not in req.args:
            return 'new'
        
        action = req.args['action']
        action_changes = {}
        
        for controller in self._get_action_controllers(req, ticket, action):
            action_changes.update(controller.get_ticket_changes(req, \
                    ticket, action))
        
        return 'status' in action_changes and action_changes['status'] or \
                ticket['status']
        
    def _get_action_controllers(self, req, ticket, action):
        
        for controller in TicketSystem(self.env).action_controllers:
            actions = [action for weight, action in
                       controller.get_ticket_actions(req, ticket)]
            if action in actions:
                yield controller
